/* @(#)SerialConnParDialog.java
 *
 * vfstagecontroller java package: A simple way to drive a microscope serial
 * controlled stage.
 * Copyright 2003(C) Policlinico Universitario di Udine
 * Written by: Federico Viel
            vielfede@libero.it

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package vfstagecontroller;

import java.awt.*;
import java.awt.event.*;
/**
 * <p>Title: SerialConnParDialog</p>
 * <p>Description: A dialog used to set connection parameter with serial port. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Policlinico Universitario di Udine</p>
 * @author Federico Viel
 * @version 1.0
 */

public class SerialConnParDialog extends Dialog {
  private Button ok_button = new Button();
  private Button cancel_button = new Button();
  private Label label1 = new Label();
  private Label label2 = new Label();
  private Label label3 = new Label();
  private Label label4 = new Label();
  private Label label5 = new Label();
  private Label label6 = new Label();
  private Choice PortNameCombo = new Choice();
  private Choice BaudRateCombo = new Choice();
  private Choice FlowControlCombo = new Choice();
  private Choice DataBitsCombo = new Choice();
  private Choice ParityCombo = new Choice();
  private Choice StopBitsCombo = new Choice();


  private SerialParameters connPar;
  private String connPortName[];

  public SerialConnParDialog (Frame frame, String title, boolean modal,SerialParameters p, String cpn[]) {

   super(frame,"Serial stage Connection parameters",modal);
   connPortName=cpn;
   connPar=p;
   initDialog();
  }

  /** Select current serial parameter in combo boxes
   * No control on parameters is effectuated
   */
  public void initCombos(){
    BaudRateCombo.select(connPar.getBaudRateString());
    FlowControlCombo.select(connPar.getFlowControlInString());
    DataBitsCombo.select(connPar.getDatabitsString());
    ParityCombo.select(connPar.getParityString());
    StopBitsCombo.select(connPar.getStopbitsString());
    PortNameCombo.select(connPar.getPortName());
  }
  /** Init dialog window controls
   */

  private void initDialog() {
    ok_button.setLabel("Ok");
    ok_button.setBounds(new Rectangle(104, 145, 75, 27));
    ok_button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ok_button_actionPerformed(e);
      }
    });
    this.setLayout(null);
    cancel_button.setLabel("Cancel");
    cancel_button.setBounds(new Rectangle(189, 145, 75, 27));
    cancel_button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        cancel_button_actionPerformed(e);
      }
    });
    label1.setText("Port name");
    label1.setBounds(new Rectangle(18, 41, 68, 17));
    label2.setText("Baud Rate");
    label2.setBounds(new Rectangle(18, 72, 61, 17));
    label3.setText("Flow Control");
    label3.setBounds(new Rectangle(18, 103, 74, 17));
    label4.setText("Data Bits");
    label4.setBounds(new Rectangle(201, 44, 69, 17));
    label5.setText("Parity");
    label5.setBounds(new Rectangle(201, 73, 76, 17));
    label6.setText("Stop Bits");
    label6.setBounds(new Rectangle(201, 99, 92, 17));
    PortNameCombo.setBounds(new Rectangle(96, 44, 89, 21));
    BaudRateCombo.setBounds(new Rectangle(96, 73, 89, 21));
    FlowControlCombo.setBounds(new Rectangle(96, 100, 89, 21));
    DataBitsCombo.setBounds(new Rectangle(293, 42, 65, 21));
    ParityCombo.setBounds(new Rectangle(293, 71, 65, 21));
    StopBitsCombo.setBounds(new Rectangle(293, 99, 65, 21));
    this.add(label4, null);
    this.add(label5, null);
    this.add(label6, null);
    this.add(FlowControlCombo, null);
    this.add(DataBitsCombo, null);
    this.add(StopBitsCombo, null);
    this.add(ParityCombo, null);
    this.add(label3, null);
    this.add(label2, null);
    this.add(label1, null);
    this.add(BaudRateCombo, null);
    this.add(PortNameCombo, null);
    this.add(ok_button, null);
    this.add(cancel_button, null);
    //Init com port combo control
    for (int i=0;i<connPortName.length;i++)
      if (connPortName[i]!=null)
        PortNameCombo.addItem(connPortName[i]);
    //Init Baud Rate combo control
    BaudRateCombo.addItem("19200");
    BaudRateCombo.addItem("9600");
    BaudRateCombo.addItem("4800");
    BaudRateCombo.addItem("2400");
    BaudRateCombo.addItem("1200");
    BaudRateCombo.addItem("600");
    BaudRateCombo.addItem("300");
    //Init Control combo control
    FlowControlCombo.addItem("None");
    FlowControlCombo.addItem("Xon/Xoff In");
    FlowControlCombo.addItem("Xon/Xoff Out");
    FlowControlCombo.addItem("RTS/CTS In");
    FlowControlCombo.addItem("RTS/CTS Out");
    //Init Data Bits combo control
    DataBitsCombo.addItem("5");
    DataBitsCombo.addItem("6");
    DataBitsCombo.addItem("7");
    DataBitsCombo.addItem("8");
    //Init Parity combo control
    ParityCombo.addItem("None");
    ParityCombo.addItem("Odd");
    ParityCombo.addItem("Even");
    //Init Stop Bits combo control
    StopBitsCombo.addItem("1");
    StopBitsCombo.addItem("1.5");
    StopBitsCombo.addItem("2");

    this.setSize(new Dimension(413, 199));
    Point p=this.getParent().getLocation();
    p.translate(40,60);
    this.setLocation(p);
    //Select from combos current serial connection parameters
    initCombos();
  }


  void ok_button_actionPerformed(ActionEvent e) {
     connPar.setBaudRate((String)BaudRateCombo.getSelectedItem());
     connPar.setDatabits((String)DataBitsCombo.getSelectedItem());
     connPar.setFlowControlIn((String)FlowControlCombo.getSelectedItem());
     connPar.setParity((String)ParityCombo.getSelectedItem());
     connPar.setFlowControlOut((String)FlowControlCombo.getSelectedItem());
     connPar.setStopbits((String)StopBitsCombo.getSelectedItem());
     connPar.setPortName((String)PortNameCombo.getSelectedItem());
     this.setVisible(false);
   }

   void cancel_button_actionPerformed(ActionEvent e) {
     this.setVisible(false);
     initCombos();
  }
}