/* @(#)StageCommands.java
 *
 * vfstagecontroller java package: A simple way to drive a microscope serial
 * controlled stage.
 * Copyright 2003(C) Policlinico Universitario di Udine
 * Written by: Federico Viel
            vielfede@libero.it

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package vfstagecontroller;
/**
 * <p>Title: StageCommands Class</p>
 * <p>Description: This class (HAVE TO) contains all string (ASCII) codify of stage commands,
 *  and building methods necessary to generate consistent (well formed) stage commands
 *  for a serial stage controlled.
 *  Commands have to be sended to stage controller to control it.
 *  A tipical serial command has the following syntax:
 *  command[#par1][$par2]T
 *  where $ is a parameters separeator, # is command separator and T is a command terminator
 *  Typically $ = # = " " and T=Return (i.e. "\n")
 *  builtCommand functions permit to construct automatically a well formed command.
 *  The class here implemented is built to control the PRIOR OPTI SCAN STAGES!
 *  It is able to control X,Y,Z axis ONLY! Nothing deny the possibility to extend the class to
 *  control shutters or filters, but this imply to modify the StageController class.
 *  Changing this class maight allow the user to control other serial controller.
 *  In this case you have to pay attention on separator and special characters specification i.e. ,#,T
 *  loadCommandFromFile (will) allow user to load a different stage protocol from file.</p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Policlinio Universitario di Udine</p>
 * @see StageController
 * @author Federico Viel
 * @version PRIOR OPTI SCAN STAGES 1.0
 */
public class StageCommands {
  /**Stage controller drived by this class*/
  static String STAGE_NAME="PRIOR Opti-Scan";

  //Measure Units
  /**X and Y axis units*/
  static final String XYUnits="Microns";
  /**Z axis units*/
  static final String ZUnits=" Tens of Microns";


  //STAGE COMMANDS and constants

  /***/
  static final String GET_STATUS="$";

  /**Command mode selector*/
  static final String MODE="comp";

  /**Default parameters separator*/
  static final String PAR_SEPARATOR=" ";
  /**Default command separator*/
  static final String CMD_SEPARATOR=" ";

  /**Stop stage command*/
  static final String STOP_STAGE="i";
  /**Goes to absolute position x,y[,z]*/
  static final String MOVE_TO_ABS_POS="g";
  /**Gets x,y,z stage coords*/
  static final String GET_COORD="";
  /**Moves Z to absolute position z*/
  static final String MOVE_Z_TO_ABS_POS="v";
  /**Command (character) terminator:Ex 13=Carriage return*/
  static final  char CMD_TERMINATOR=13;
  /**Sets up absolute stage position*/
  static final String SET_ABS_POS="p";
  /**Stage X-Y joystick backlash command control*/
  static final String STAGE_BACKLASH_J="blsj";
  /**Stage Z joystick backlash command control*/
  static final String FOCUS_BACKLASH_J="blzj";
  /**Stage X-Y backlash command control*/
  static final String STAGE_BACKLASH_H="blsh";
  /**Stage Z backlash command control*/
  static final String Z_BACKLASH_H="blzh";
  /**X-Y Speed control command*/
  static final String XY_SPEED="sms";
  /**X-Y Speed under joistick control command*/
  static final String XY_JOY_SPEED="o";
  /**X-Y Acceleration control command*/
  static final String XY_ACC="sas";
  /**Z Speed control command*/
  static final String Z_SPEED="smz";
  /**Z Speed under joistick control command*/
  static final String Z_JOY_SPEED="of";
  /**Z Acceleration control command*/
  static final String Z_ACC="saz";
  /**Sets/gets X step size*/
  static final String X_STEP_SIZE="x";
  /**Sets/gets Y step size*/
  static final String Y_STEP_SIZE="x";
  /**Sets/gets Z step size*/
  static final String Z_STEP_SIZE="c";
  /**Move stage forward (by y steps)*/
  static final String MOVE_FWD="f";
  /**Move stage backward (by y steps)*/
  static final String MOVE_BWD="b";
  /**Move stage left (by y steps)*/
  static final String MOVE_LEFT="l";
  /**Move stage right (by y steps)*/
  static final String MOVE_RIGHT="r";
  /**Move stage up (by y steps)*/
  static final String MOVE_Z_UP="d";
  /**Move stage down (by y steps)*/
  static final String MOVE_Z_DOWN="u";


  /**Stage X dimension*/
  static final long X_LIMIT=76000;
  /**Stage Y dimension*/
  static final long Y_LIMIT=52000;


  /**X-Y acceleration range MAX*/
  static final int MAX_XY_ACC=100;
  /**X-Y acceleration range MIN*/
  static final int MIN_XY_ACC=4;
  /**Z acceleration range MAX*/
  static final int MAX_Z_ACC=100;
  /**Z acceeration range MIN*/
  static final int MIN_Z_ACC=4;


  /**X-Y speed range MAX*/
  static final int MAX_XY_SPEED=100;
  /**X-Y speed range MIN*/
  static final int MIN_XY_SPEED=1;
  /**Z speed range MAX*/
  static final int MAX_Z_SPEED=100;
  /**Z speed range MIN*/
  static final int MIN_Z_SPEED=1;



  /*Deprecated commands and constants*/

  /**Stage speed changer*/
static final String STAGE_SPEED="VS";

/**Stage max settable speed*/
static final int STAGE_MIN_SPEED=-8000;
/**Stage min settable speed*/
  static final int STAGE_MAX_SPEED=+8000;

  public StageCommands() {
  }
  /** Build  a no-parameters instruction. May be this function is useles!
   * @param cmd command
   * @return well formed command
   */
  static public String builtCommand(String cmd){
    String command=new String (cmd);
    return command;
  }
  /** Build a single parameters instruction.
   * @param cmd command
   * @param p1 first argument
   * @return well formed 1 argument command
   */
  static public String builtCommand(String cmd, long p1){
    String command=new String (cmd+CMD_SEPARATOR+String.valueOf(p1));
    return command;
  }
  /** Build a double parameters instruction.
   * @param cmd command
   * @param p1 first argument
   * @param p2 second argument
   * @return well formed 2 argument command
   */
  static public String builtCommand(String cmd, long p1, long p2){
    String command=new String (cmd+CMD_SEPARATOR+String.valueOf(p1)+PAR_SEPARATOR+String.valueOf(p2));
    return command;
  }
  /** Build a triple parameters instruction.
   * @param cmd command
   * @param p1 first argument
   * @param p2 second argument
   * @param p3 third argument
   * @return well formed 3 argument command
   */
  static public String builtCommand(String cmd, long p1, long p2, long p3){
    String command=new String (cmd+CMD_SEPARATOR+String.valueOf(p1)+PAR_SEPARATOR+String.valueOf(p2)+PAR_SEPARATOR+String.valueOf(p3));
    return command;
  }
  /** Load a new stage protocol from file. Protocol means: stage commands, terminator, dimension etc.
  * @param file file name where data is contained. File contains one commands for each line and each line has the following format:
  * <COMMAND NAME> <COMMAND STRING>
  * Where <COMMAND NAME> is the name of the constant in the class.
  *  Ex.  GET_COORD gigi
  * @return true if load process has success false otherwise
  * @deprecated To be implemented!
  */
  static public boolean loadCommandFromFile(String file){
  return false;
  }

}