/* @(#)StageCtrlDialog.java
 *
 * vfstagecontroller java package: A simple way to drive a microscope serial
 * controlled stage.
 * Copyright 2003(C) Policlinico Universitario di Udine
 * Written by: Federico Viel
            vielfede@libero.it

 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package vfstagecontroller;
import javax.comm.*;
import java.lang.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;

/**
 * <p>Title: Stage control dialog </p>
 * <p>Description: A simple dialog to control basic function stage: steps size, steps, etc. </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Policlinico Universitario di Udine</p>
 * @author Federico Viel
 * @version 1.0
 */

public class StageCtrlDialog extends Frame {

  private int c_bookmark_N;
  private StageController stController;

  private static TextArea commandArea = new TextArea();


  private TextField xField = new TextField();
  private TextField yField = new TextField();
  private TextField zField = new TextField();

  private Button stageButtonLeft = new Button();
  private Button stageButtonRight = new Button();
  private Button stageButtonUp = new Button();
  private Button stageButtonDown = new Button();
  private Button zAxisButtonUp = new Button();
  private Button zAxisButtonDown = new Button();
  private Button updateButton = new Button();
  private Button searchLimits = new Button();
  private Button PosResetButton = new Button();
  private Button stepSize = new Button();
  private Button connectButton = new Button();
  private Button disconnectButton = new Button();
  private Button connParButton = new Button();


  private Checkbox BacklashOnOff = new Checkbox();

  private Scrollbar stageSpeedScrollBar = new Scrollbar();
  private Scrollbar stageAccScrollBar = new Scrollbar();
  private Scrollbar zSpeedScrollBar = new Scrollbar();
  private Scrollbar zAccScrollBar = new Scrollbar();

  private static SerialConnParDialog serialCtrlPanel;
  private Label label1 = new Label();
  private Label label2 = new Label();
  private Label label3 = new Label();
  private Label label4 = new Label();
  private Label label5 = new Label();
  private Label label6 = new Label();
  private Label label7 = new Label();


  private MenuBar menuBar = new MenuBar();
  private Menu fileMenu = new Menu();
  private MenuItem loadMenuItem = new MenuItem();
  private Menu bookmarkMenu = new Menu();
  private MenuItem addBookmarkMenuItem = new MenuItem();
  private Menu helpMenu = new Menu();
  private MenuItem aboutMenuItem = new MenuItem();
  private MenuItem exitMenuItem = new MenuItem();

  private boolean runAsPlugin;

  private AboutDialog aboutDlg;

  /** Construct a new Stage controller dialog and prompt for stage controller parameters
   */
  public StageCtrlDialog() {
    this(false);
  }
  /** Construct a new Stage controller dialog and prompt for stage controller parameters
   * @param plugin select if the window have to work as plugin (i.e. from another application) or not.
   */
  public StageCtrlDialog(boolean plugin) {
    runAsPlugin=plugin;
    //Inti bookmark number
    c_bookmark_N=0;
    //Create an about dialog window
    aboutDlg=new AboutDialog(this,"About",StageCommands.STAGE_NAME);
    //Create a new stage controller instance WITH STANDARD SERIAL CONNECTION PARAMETERS
    stController= new StageController(commandArea);
    //Create serial connection parameters dialog (useful to change connection parameters).  SERIAL CONNECTION PARAMETERS got from stController are passed to consturctor.
    serialCtrlPanel=new SerialConnParDialog(this,"Stage connection parameters",true,stController.getConnectionParameters(),stController.getCommPortFounded());

    initDialog();
    show();
    aboutDlg.show();
    serialCtrlPanel.show();
  }
  /**
   * Launch the application
   * @param args default main parameters: ignored!
   */
  public static void main(String[] args) {
    StageCtrlDialog dialogo=new StageCtrlDialog();

  }

  private void initDialog() {
    setLayout(null);
    this.setBackground(SystemColor.text);
    this.setSize(new Dimension(500, 400));
    this.setTitle("Java serial stage controlled");
    this.setMenuBar(menuBar);
    this.setResizable(false);

    commandArea.setEditable(false);
    commandArea.setBounds(new Rectangle(329, 70, 121, 227));
    connectButton.setLabel("Connect");
    connectButton.setBounds(new Rectangle(10, 316, 104, 27));
    connectButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        connectButton_actionPerformed(e);
      }
    });

    disconnectButton.setLabel("Disconnect");
    disconnectButton.setBounds(new Rectangle(117, 316, 101, 27));
    disconnectButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        disconnectButton_actionPerformed(e);
      }
    });
    connParButton.setLabel("Connection Parameters");
    connParButton.setBounds(new Rectangle(11, 348, 208, 27));
    connParButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        connParButton_actionPerformed(e);
      }
    });
    xField.setBounds(new Rectangle(74, 116, 65, 21));
    xField.addKeyListener(new KeyAdapter(){
      public void keyTyped(KeyEvent e){
       xyfield_keyTyped(e);
      }
    });
    yField.setBounds(new Rectangle(74, 139, 66, 21));
    yField.addKeyListener(new KeyAdapter(){
     public void keyTyped(KeyEvent e){
       xyfield_keyTyped(e);
     }
    });
    zField.setBounds(new Rectangle(237, 129, 67, 21));
    zField.addKeyListener(new KeyAdapter(){
     public void keyTyped(KeyEvent e){
       zfield_keyTyped(e);
     }
    });
    stageButtonLeft.setLabel("Left");
    stageButtonLeft.setBounds(new Rectangle(10, 117, 65, 44));
    stageButtonLeft.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stageButtonLeft_actionPerformed(e);
      }
    });
    stageButtonRight.setLabel("Right");
    stageButtonRight.setBounds(new Rectangle(141, 117, 68, 45));
    stageButtonRight.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stageButtonRight_actionPerformed(e);
      }
    });
    stageButtonUp.setLabel("Up");
    stageButtonUp.setBounds(new Rectangle(74, 73, 67, 45));
    stageButtonUp.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stageButtonUp_actionPerformed(e);
      }
    });
    stageButtonDown.setLabel("Down");
    stageButtonDown.setBounds(new Rectangle(74, 161, 66, 47));
    stageButtonDown.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stageButtonDown_actionPerformed(e);
      }
    });
    zAxisButtonUp.setLabel("Up");
    zAxisButtonUp.setBounds(new Rectangle(237, 70, 68, 60));
    zAxisButtonUp.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        zAxisButtonUp_actionPerformed(e);
      }
    });
    zAxisButtonDown.setLabel("Down");
    zAxisButtonDown.setBounds(new Rectangle(237, 150, 68, 60));
    zAxisButtonDown.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        zAxisButtonDown_actionPerformed(e);
      }
    });



    updateButton.setLabel("Update");
    updateButton.setBounds(new Rectangle(10, 250, 104, 28));
    updateButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        updateButton_actionPerformed(e);
      }
    });

    BacklashOnOff.setLabel("Backlash on/off");
    BacklashOnOff.setBounds(new Rectangle(12, 217, 205, 25));
    BacklashOnOff.addItemListener(new java.awt.event.ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        BacklashOnOff_itemStateChanged(e);
      }
    });

    stepSize.setLabel("Steps size");
    stepSize.setBounds(new Rectangle(117, 250, 101, 28));
    stepSize.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stepSize_actionPerformed(e);
      }
    });
    stageSpeedScrollBar.setBounds(new Rectangle(303, 334, 70, 17));
    stageSpeedScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        stageSpeedScrollBar_adjustmentValueChanged(e);
      }
    });
    stageSpeedScrollBar.setMaximum(StageCommands.MAX_XY_SPEED);
    stageSpeedScrollBar.setMinimum(StageCommands.MIN_XY_SPEED);

    stageSpeedScrollBar.setOrientation(0);
    zSpeedScrollBar.setOrientation(0);
    zSpeedScrollBar.setBounds(new Rectangle(382, 334, 69, 20));
    zSpeedScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        zSpeedScrollBar_adjustmentValueChanged(e);
      }
    });
    zSpeedScrollBar.setMaximum(StageCommands.MAX_Z_SPEED);
    zSpeedScrollBar.setMinimum(StageCommands.MIN_Z_SPEED);

    label2.setText("Speed");
    label2.setBounds(new Rectangle(255, 334, 37, 23));
    label3.setFont(new java.awt.Font("Dialog", 1, 15));
    label3.setText("Z");
    label3.setBounds(new Rectangle(413, 309, 21, 21));
    label4.setBounds(new Rectangle(255, 352, 43, 23));
    label4.setText("Accel.");
    label5.setText("Serial connection ch.");
    label5.setBounds(new Rectangle(328, 50, 132, 23));
    label1.setText("X-Y axis");
    label1.setBounds(new Rectangle(10, 50, 70, 19));
    label6.setText("Z axis");
    label6.setBounds(new Rectangle(237, 50, 34, 17));
    stageAccScrollBar.setBounds(new Rectangle(303, 354, 70, 18));
    stageAccScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        stageAccScrollBar_adjustmentValueChanged(e);
      }
    });
    stageAccScrollBar.setMaximum(StageCommands.MAX_XY_ACC);
    stageAccScrollBar.setMinimum(StageCommands.MIN_XY_ACC);
    stageAccScrollBar.setOrientation(0);


    searchLimits.setLabel("Search limits");
    searchLimits.setBounds(new Rectangle(10, 283, 104, 27));
    searchLimits.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        searchLimits_actionPerformed(e);
      }
    });
    PosResetButton.setLabel("Pos. reset");
    PosResetButton.setBounds(new Rectangle(117, 283, 101, 27));
    PosResetButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        PosResetButton_actionPerformed(e);
      }
    });
    zAccScrollBar.setOrientation(0);
    zAccScrollBar.setBounds(new Rectangle(382, 356, 69, 16));
    zAccScrollBar.addAdjustmentListener(new java.awt.event.AdjustmentListener() {
      public void adjustmentValueChanged(AdjustmentEvent e) {
        zAccScrollBar_adjustmentValueChanged(e);
      }
    });
    zAccScrollBar.setMaximum(StageCommands.MAX_Z_ACC);
    zAccScrollBar.setMinimum(StageCommands.MIN_Z_ACC);

    label7.setBounds(new Rectangle(326, 308, 37, 21));
    label7.setText("X/Y");
    label7.setFont(new java.awt.Font("Dialog", 1, 15));

    loadMenuItem.setLabel("Load...");
    loadMenuItem.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        loadMenuItem_actionPerformed(e);
      }
    });
    fileMenu.setLabel("File");
    bookmarkMenu.setLabel("Bookmark");
    addBookmarkMenuItem.setLabel("Add");
    addBookmarkMenuItem.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addBookmarkMenuItem_actionPerformed(e);
      }
    });
    helpMenu.setLabel("Help");
    aboutMenuItem.setLabel("About");
    aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        aboutMenuItem_actionPerformed(e);
      }
    });
    exitMenuItem.setLabel("Exit");
    exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exitMenuItem_actionPerformed(e);
      }
    });

    this.add(stageButtonUp, null);
    this.add(BacklashOnOff, null);
    this.add(stageButtonRight, null);
    this.add(stageButtonLeft, null);
    this.add(xField, null);
    this.add(yField, null);
    this.add(stageButtonDown, null);
    this.add(updateButton, null);
    this.add(connectButton, null);
    this.add(connParButton, null);
    this.add(disconnectButton, null);
    this.add(stepSize, null);
    this.add(searchLimits, null);
    this.add(PosResetButton, null);
    this.add(stageSpeedScrollBar, null);
    this.add(label2, null);
    this.add(label3, null);
    this.add(zSpeedScrollBar, null);
    this.add(zAccScrollBar, null);
    this.add(stageAccScrollBar, null);
    this.add(label7, null);
    this.add(label4, null);
    this.add(commandArea, null);
    this.add(label5, null);
    this.add(label1, null);
    this.add(zField, null);
    this.add(zAxisButtonDown, null);
    this.add(zAxisButtonUp, null);
    this.add(label6, null);
    menuBar.add(fileMenu);
    menuBar.add(bookmarkMenu);
    menuBar.add(helpMenu);
    fileMenu.add(loadMenuItem);
    fileMenu.addSeparator();
    fileMenu.add(exitMenuItem);
    helpMenu.add(aboutMenuItem);

    disbleControls();


    this.addWindowListener(new WindowAdapter(){
        public void windowClosing(WindowEvent e){
         stController.closeConnection();
         dispose();
        }
        public void windowClosed(WindowEvent e){
        if (!runAsPlugin)
          System.exit(0);
        }
    });
  }

private void enableControls(){
  bookmarkMenu.setEnabled(true);
  stageButtonLeft.setEnabled(true);
  stageButtonRight.setEnabled(true);
  stageButtonUp.setEnabled(true);
  stageButtonDown.setEnabled(true);
  zAxisButtonUp.setEnabled(true);
  zAxisButtonDown.setEnabled(true);
  xField.setEnabled(true);
  yField.setEnabled(true);
  zField.setEnabled(true);
  BacklashOnOff.setEnabled(true);
  updateButton.setEnabled(true);
  stepSize.setEnabled(true);
  searchLimits.setEnabled(true);
  PosResetButton.setEnabled(true);
  disconnectButton.setEnabled(true);
  connectButton.setEnabled(false);
  connParButton.setEnabled(false);
  stageSpeedScrollBar.setEnabled(true);
  stageAccScrollBar.setEnabled(true);
  zSpeedScrollBar.setEnabled(true);
  zAccScrollBar.setEnabled(true);
}
private void disbleControls(){
  bookmarkMenu.setEnabled(false);
  stageButtonLeft.setEnabled(false);
  stageButtonRight.setEnabled(false);
  stageButtonUp.setEnabled(false);
  stageButtonDown.setEnabled(false);
  zAxisButtonUp.setEnabled(false);
  zAxisButtonDown.setEnabled(false);
  xField.setEnabled(false);
  yField.setEnabled(false);
  zField.setEnabled(false);
  BacklashOnOff.setEnabled(false);
  updateButton.setEnabled(false);
  stepSize.setEnabled(false);
  searchLimits.setEnabled(false);
  PosResetButton.setEnabled(false);
  disconnectButton.setEnabled(false);
  connectButton.setEnabled(true);
  connParButton.setEnabled(true);

  stageSpeedScrollBar.setEnabled(false);
  stageAccScrollBar.setEnabled(false);
  zSpeedScrollBar.setEnabled(false);
  zAccScrollBar.setEnabled(false);
}
  void disconnectButton_actionPerformed(ActionEvent e) {
    stController.closeConnection();
    disbleControls();
  }

  void updateFields(){
    long pos=stController.getCurrentPos(stController.X_COOR);
    xField.setText(String.valueOf(pos));
    pos=stController.getCurrentPos(stController.Y_COOR);
    yField.setText(String.valueOf(pos));
    pos=stController.getCurrentPos(stController.Z_COOR);
    zField.setText(String.valueOf(pos));
    stageSpeedScrollBar.setValue(stController.getXYSpeed());
    stageAccScrollBar.setValue(stController.getXYAcc());
    zAccScrollBar.setValue(stController.getZAcc());
    zSpeedScrollBar.setValue(stController.getZSpeed());
    BacklashOnOff.setState(stController.getBacklashStageStat());
  }

  void updateBookmarks(){
    bookmarkMenu.setEnabled(false);
    bookmarkMenu.removeAll();
    bookmarkMenu.add(addBookmarkMenuItem);
    bookmarkMenu.addSeparator();
    for (int i=0;i<stController.N_BOOKMARK;i++){
      if (!stController.getBookmarkName(i).equals("")){
        MenuItem mi=new MenuItem(stController.getBookmarkName(i));
        mi.addActionListener(new ActionListener(){
          public void actionPerformed(ActionEvent e){
            stController.moveStageTo(stController.getBookmarkCoord(e.getActionCommand(),stController.X_COOR),stController.getBookmarkCoord(e.getActionCommand(),stController.Y_COOR),stController.getBookmarkCoord(e.getActionCommand(),stController.Z_COOR));
          }
        });
        bookmarkMenu.add(mi);
      }
    }
    bookmarkMenu.setEnabled(true);
  }
  void updateButton_actionPerformed(ActionEvent e) {
   updateFields();
  }

  void connectButton_actionPerformed(ActionEvent e) {

    try{
       if (!stController.openConnection()){
         MessageDialog m=new MessageDialog(this,"Error","Unable to open stage connection. Please check connection paramters and try again!");
         m.show();
         stController.closeConnection();
       }else{
         enableControls();
         updateFields();
         stController.initBookmarks();
         updateBookmarks();
       }
    }catch(Exception ee){
    }
  }

  void connParButton_actionPerformed(ActionEvent e) {
    Point p=serialCtrlPanel.getParent().getLocation();
    p.translate(40,60);
    serialCtrlPanel.setLocation(p);
     serialCtrlPanel.show();
  }

  void stageButtonUp_actionPerformed(ActionEvent e) {
    stController.moveXYStep(StageCommands.MOVE_FWD);
    updateFields();
  }

  void stageButtonLeft_actionPerformed(ActionEvent e) {
    stController.moveXYStep(StageCommands.MOVE_LEFT);
    updateFields();
  }

  void stageButtonRight_actionPerformed(ActionEvent e) {
    stController.moveXYStep(StageCommands.MOVE_RIGHT);
    updateFields();
  }

  void stageButtonDown_actionPerformed(ActionEvent e) {
    stController.moveXYStep(StageCommands.MOVE_BWD);
    updateFields();
  }

  void BacklashOnOff_itemStateChanged(ItemEvent e) {
    if (e.SELECTED==e.getStateChange())
      stController.enableStageBackLash(true);
    else
      stController.enableStageBackLash(false);
  }

  void stepSize_actionPerformed(ActionEvent e) {
    StepSizeDialog ssdlg=new StepSizeDialog(this,"Step size control dialog",true);
    ssdlg.setFields(stController.getXStepSize(),stController.getYStepSize(),stController.getZStepSize());
    ssdlg.show();
    if (ssdlg.status)
      stController.setXYZStepSize(ssdlg.getXSField(),ssdlg.getYSField(),ssdlg.getZSField());
    return;

  }

  void xFieldValueChanged(long nPos){
    stController.moveStageTo(nPos,stController.getCurrentPos(stController.Y_COOR));
  }

  void zAxisButtonUp_actionPerformed(ActionEvent e) {
    stController.moveZStep(StageCommands.MOVE_Z_UP);
    updateFields();
  }

  void zAxisButtonDown_actionPerformed(ActionEvent e) {
    stController.moveZStep(StageCommands.MOVE_Z_DOWN);
    updateFields();
  }

  void stageSpeedScrollBar_adjustmentValueChanged(AdjustmentEvent e) {
    stController.setXYSpeed(stageSpeedScrollBar.getValue());
  }

  void stageAccScrollBar_adjustmentValueChanged(AdjustmentEvent e) {
     stController.setXYAcc(stageAccScrollBar.getValue());
  }

  void searchLimits_actionPerformed(ActionEvent e) {
    MessageDialog p=new MessageDialog(this,"Warning","Now Stage Controller will move the stage to find out phisical limit. Wolud you like to continue?",MessageDialog.OK+MessageDialog.CANCEL);
    p.show();
    if (p.rValue==MessageDialog.OK){
      MessageDialog d=new MessageDialog(this,"Warning","Wait while the program is searching phisical stage limits... Press HALT to stop the stage?",new String("HALT"));
      stController.setDialogToHide(d);
      Thread t=new Thread(stController);
      t.start();
      d.show();
      if (d.rValue==MessageDialog.OK)
        stController.stopStage();
      this.updateFields();
    }
  }

  void PosResetButton_actionPerformed(ActionEvent e) {
    this.bookmarkMenu.setEnabled(false);
    stController.setAbsPos(0,0,0);
    stController.initBookmarks();
    this.updateFields();
    this.updateBookmarks();
  }

  void zSpeedScrollBar_adjustmentValueChanged(AdjustmentEvent e) {
    stController.setZSpeed(zSpeedScrollBar.getValue());
  }

  void zAccScrollBar_adjustmentValueChanged(AdjustmentEvent e) {
    stController.setZAcc(zAccScrollBar.getValue());

  }


void addBookmark(){
  long a,b,c;
    a=stController.getCurrentPos(stController.X_COOR);
    b=stController.getCurrentPos(stController.Y_COOR);
    c=stController.getCurrentPos(stController.Z_COOR);
    String s=new String("bookrmark"+c_bookmark_N++);
    stController.addBookmark(s,a,b,c);
    updateFields();
    updateBookmarks();
}


  void addBookmarkMenuItem_actionPerformed(ActionEvent e) {
    this.bookmarkMenu.setEnabled(false);
    addBookmark();
  }

  void exitMenuItem_actionPerformed(ActionEvent e) {
    System.exit(0);
  }

  void loadMenuItem_actionPerformed(ActionEvent e) {
    MessageDialog m=new MessageDialog(this,"Error","Load new stage configuration will be available in next versions!");
    m.show();
  }

  void aboutMenuItem_actionPerformed(ActionEvent e) {
    Point p=aboutDlg.getParent().getLocation();
    p.translate(40,60);
    aboutDlg.setLocation(p);
    aboutDlg.show();
  }
  public void zfield_keyTyped(KeyEvent e){
       if (e.getKeyChar()=='\n') {
         stController.moveZAxisTo(Long.parseLong(zField.getText()));
       }
  }
  public void xyfield_keyTyped(KeyEvent e){
    if (e.getKeyChar()=='\n') {
        long x,y;
        x=Long.parseLong(xField.getText());
        y=Long.parseLong(yField.getText());
        stController.moveStageTo(x,y);
        }
  }
}